import functools
import pymysql
import threading
from pymysql.cursors import DictCursor
from .__env_config import config as default_config
from functools import partial


class Database:
    def __init__(self, config):
        self.config = config or default_config['database']

    def __enter__(self):
        port = 3306
        if 'port' in self.config.keys():
            port = self.config['port']
        self.__connect = pymysql.connect(host=self.config['host'], user=self.config['user'], port=port,
                                         password=str(self.config['password']), database=self.config['dbname'])
        self.__cursor = self.connect.cursor(DictCursor)
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.cursor.close()
        self.connect.close()

    @property
    def connect(self):
        return self.__connect

    @property
    def cursor(self):
        return self.__cursor


__local__ = threading.local()


def read(func=None, config=None, one=False):
    if func is None:
        return functools.partial(read, config=config, one=one)

    def execute(db, sql):
        db.cursor.execute(sql)
        result = db.cursor.fetchall()
        if one:
            return result[0] if result else None
        else:
            return result

    @functools.wraps(func)
    def wraps(*args, **kwargs):
        sql = func(*args, **kwargs)
        if hasattr(__local__, 'db'):
            return execute(__local__.db, sql)
        else:
            with Database(config) as db:
                return execute(db, sql)
    return wraps


def write(func=None, config=None):
    if func is None:
        return functools.partial(write, func=func, config=config)

    def execute(db, sqls):
        if isinstance(sqls, list):
            for sql in sqls:
                db.cursor.execute(sql)
        else:
            db.cursor.execute(sqls)

    @functools.wraps(func)
    def wraps(*args, **kwargs):
        sqls = func(*args, **kwargs)
        if hasattr(__local__, 'db'):
            execute(__local__.db, sqls)
        else:
            with Database(config) as db:
                try:
                    execute(db, sqls)
                    db.connect.commit()
                except Exception as e:
                    db.connect.rollback()
                    raise e
    return wraps


def transaction(func=None, config=None):
    if func is None:
        return functools.partial(transaction, func=func, config=config)

    @functools.wraps(func)
    def wraps(*args, **kwargs):
        if hasattr(__local__, 'db'):
            return func(*args, **kwargs)
        with Database(config) as db:
            __local__.db = db
            try:
                result = func(*args, **kwargs)
                db.connect.commit()
                return result
            except Exception as e:
                db.connect.rollback()
                raise e
            finally:
                del __local__.db
    return wraps


